import { generateTokens, tokenize } from "./tokenizer.js";
const ruleTypes = ["ALLOW", "DISALLOW", "CRAWL_DELAY"];
/**
 * Grammar:
 *
 * RobotsTxt
 *  : ConfigList
 *  | Sitemap
 *  ;
 *
 * ConfigList:
 *  : Config
 *  | ConfigList Config
 *  ;
 *
 * Config:
 *  : UserAgentList
 *  | RuleList
 *  ;
 *
 * UserAgentList
 *  : UserAgent
 *  | UserAgentList UserAgent
 *  ;
 *
 * RuleList
 *  : Rule
 *  | RuleList Rule
 *  ;
 */
export function parse(txt) {
    const lookahead = createLookahead(txt);
    return createRobotsTxt(lookahead);
}
function createRobotsTxt(lookahead) {
    const robotsTxt = {
        type: "RobotsTxt",
        configs: [],
        sitemap: undefined,
    };
    if (!lookahead.hasMore()) {
        return robotsTxt;
    }
    while (lookahead.hasMore()) {
        switch (lookahead.current().type) {
            case "SITEMAP": {
                const sitemap = createSitemap(lookahead);
                robotsTxt.sitemap = sitemap;
                break;
            }
            case "USER_AGENT": {
                const config = createConfig(lookahead);
                robotsTxt.configs.push(config);
                break;
            }
            default: {
                const token = lookahead.current();
                throw new Error(`Unexpected token: ${token.type} at pos ${token.position}`);
            }
        }
    }
    return robotsTxt;
}
function createSitemap(lookahead) {
    eat("SITEMAP", lookahead);
    eat(":", lookahead);
    const { value } = eat("VALUE", lookahead);
    return value;
}
function createConfig(lookahead) {
    const userAgents = [];
    while (lookahead.hasMore() && lookahead.current().type === "USER_AGENT") {
        eat("USER_AGENT", lookahead);
        eat(":", lookahead);
        const { value } = eat("VALUE", lookahead);
        userAgents.push(value);
    }
    const rules = [];
    while (lookahead.hasMore() && ruleTypes.includes(lookahead.current().type)) {
        const rule = createRule(lookahead);
        rules.push(rule);
    }
    return {
        type: "Config",
        userAgents,
        rules,
    };
}
function createRule(lookahead) {
    const { type, position } = lookahead.current();
    switch (type) {
        case "ALLOW": {
            eat("ALLOW", lookahead);
            eat(":", lookahead);
            const { value } = eat("VALUE", lookahead);
            return {
                type: "AllowRule",
                value: value,
            };
        }
        case "DISALLOW": {
            eat("DISALLOW", lookahead);
            eat(":", lookahead);
            const { value } = eat("VALUE", lookahead);
            return {
                type: "DisallowRule",
                value,
            };
        }
        case "CRAWL_DELAY": {
            eat("CRAWL_DELAY", lookahead);
            eat(":", lookahead);
            const { value } = eat("VALUE", lookahead);
            return {
                type: "CrawlDelayRule",
                value,
            };
        }
        default:
            throw new Error(`Syntax error: Expected a rule at pos ${position}`);
    }
}
function eat(tokenType, lookahead) {
    const token = lookahead.current();
    if (token.type !== tokenType) {
        throw new Error(`Expected ${tokenType}, found ${token.type} at pos ${token.position}`);
    }
    lookahead.findNext();
    return token;
}
function createLookahead(txt) {
    const iterator = generateTokens(txt);
    const tokens = tokenize(txt);
    let token;
    const findNext = () => {
        const { value, done } = iterator.next();
        if (done || !value) {
            token = null;
            return null;
        }
        token = value;
        return value;
    };
    const current = () => {
        if (!token) {
            throw new Error("Unexpected EOF");
        }
        return token;
    };
    const hasMore = () => token !== null;
    findNext();
    return { current, hasMore, findNext };
}
